#!ruby -Ku

$KCODE='u'

require 'kconv'
require 'uconv'
require 'be_stat'
require 'parsearg'

BookmarkData = Struct::new("BookmarkData",:url,:name)
DirData = Struct::new("DirData",:parent,:subdirs,:bookmarks)

BOOKMARK_FILETYPE='application/x-vnd.Be-bookmark'
BOOKMARK_URL_ATTR='META:url'
BOOKMARK_TITLE_ATTR='META:title'

SCRIPT_VERSION='20000602'
SCRIPT_AUTHOR='tenmei'

def show_Version()
  
  puts(File::basename($0) + ' version ' + SCRIPT_VERSION +
       ' by ' + SCRIPT_AUTHOR)
  puts()
  
end

def show_Help()
  
  show_Version()
  puts(File::basename($0) + ' <options>')
  puts()
  puts('options:')
  puts('  --nn <Navigator bookmark file> ... set convert src')
  puts('  --np <Net+ bookmark directory> ... set convert dest')
  puts('  --help | -h                    ... show help(this message)')
  puts('  --version | -v                 ... show version')
  puts()
  
end

def read_NNFile(nn_filename)
  
  File::open(nn_filename,"r") do |file|
    ret = file.readlines("\r\r\n")
    ret.each() {|l| l = Kconv::tosjis(l).gsub!(/\r\r\n/s,"")}
    return ret
  end
  
end

def make_BookmarkList(nn_text)
  
  dir_data = DirData::new()
  
  dir_data.parent = dir_data
  dir_data.subdirs = Hash::new()
  dir_data.bookmarks = Array::new()
  
  current = dir_data
  #depth = 0
  
  nn_text.each() do |line|
    if (/<DT><H3/s =~ line)
      />([^<]+)<\/H3/s =~ line
      dirname = Uconv::sjistou8($1)
      current.subdirs[dirname] = DirData::new()
      current.subdirs[dirname].parent = current
      current = current.subdirs[dirname]
      current.subdirs = Hash::new()
      current.bookmarks = Array::new()
      #puts ' ' * depth * 2 + '[' + dirname + ']'
      #depth += 1
    elsif (/<DT><A /s =~ line)
      /HREF="([^"]+)"[^>]*>([^<]+)<\/A/s =~ line
      bk = BookmarkData::new()
      bk.url = String::new($1)
      bk.name = Uconv::sjistou8($2)
      current.bookmarks.push(bk)
      #puts ' ' * (depth + 1) * 2 + bk.name + '(' + bk.url + ')'
    elsif (/<\/DL>/s =~ line)
      current = current.parent
      #depth -= 1
    end
  end
  
  return dir_data
  
end

def output_recursive(bookmark_list)
  
  bookmark_list.bookmarks.each() do |bookmark|
    fp = File::open(bookmark.name,"w")
    fp.flush()
    fp.close()
    be_attr = BeStat::Attr::open(bookmark.name)
    be_attr.SetType(BOOKMARK_FILETYPE)
    be_attr.Write(BOOKMARK_URL_ATTR,BeStat::B_STRING_TYPE,bookmark.url)
    be_attr.Write(BOOKMARK_TITLE_ATTR,BeStat::B_STRING_TYPE,bookmark.name)
  end
  
  current = Dir::pwd()
  
  bookmark_list.subdirs.each() do |subdir,subdata|
    Dir::mkdir(subdir) if (Dir::glob(subdir).size() < 1)
    Dir::chdir(subdir)
    output_recursive(subdata)
    Dir::chdir(current)
  end
  
end

def output(basepath,bookmark_list)
  
  oldpath = Dir::pwd()
  
  Dir::chdir(basepath)
  output_recursive(bookmark_list)
  
  Dir::chdir(oldpath)
  
end

def main()
  
  parseArgs(0,"(nn&np)|h|help","hv","help","verbose","version","nn:","np:")
  
  if ($OPT_h || $OPT_help)
    show_Help()
    return
  end
  
  show_Version() if ($OPT_v || $OPT_version || $OPT_verbose)
  
  nn_text = read_NNFile($OPT_nn)
  return if (!nn_text)
  
  bookmark_list = make_BookmarkList(nn_text)
  
  output($OPT_np,bookmark_list)
  
end

def usage()
  
  show_Version()
  puts("usage:#{File::basename($0)} [options]")
  puts("type #{File::basename($0)} --help to show help")
  puts()
  
end

$USAGE='usage'

main()
